@@ -14,18 +14,32 @@ from semver import VersionInfo
 from rime_sdk.internal.backend import RIMEBackend
 from rime_sdk.internal.protobuf_parser import parse_test_run_metadata
 from rime_sdk.internal.throttle_queue import ThrottleQueue
+from rime_sdk.protos.continuous.continuous_pb2 import (
+    CreateCTInstanceRequest,
+    CreateCTInstanceResponse,
+    CTInstance,
+    CTInstanceWriteMask,
+    GetActiveCTInstanceIDRequest,
+    GetActiveCTInstanceIDResponse,
+    GetCTInstanceRequest,
+    GetCTInstanceResponse,
+    UpdateCTInstanceRequest,
+    UpdateCTInstanceResponse,
+)
 from rime_sdk.protos.image_registry.image_registry_pb2 import (
     CreateImageRequest,
     GetImageRequest,
     ListImagesRequest,
     ManagedImage,
 )
-from rime_sdk.protos.jobs.jobs_pb2 import JobMetadata, JobStatus
+from rime_sdk.protos.jobs.jobs_pb2 import JobMetadata, JobStatus, JobType
 from rime_sdk.protos.model_testing.model_testing_pb2 import (
     CustomImage,
     GetLatestLogsRequest,
     GetTestJobRequest,
     ListTestJobsRequest,
+    StartContinuousTestFromReferenceRequest,
+    StartIncrementalContinuousTestRequest,
     StartStressTestRequest,
 )
 from rime_sdk.protos.results_upload.results_upload_pb2 import (
@@ -312,7 +326,10 @@ class RIMEStressTestJob:
                 raise ValueError(e)
 
     def get_test_cases_result(self, version: Optional[str] = None) -> pd.DataFrame:
-        """Return all test cases for a given test run in a Pandas dataframe."""
+        """Return all test cases for a given test run in a Pandas dataframe.
+
+        Note: this will not work for test runs run on RIME versions <0.14.0.
+        """
         if version and not VersionInfo.isvalid(version):
             raise ValueError(f"Invalid version string: {version}")
 
@@ -354,7 +371,10 @@ class RIMEStressTestJob:
             return pd.DataFrame(all_test_cases).drop(columns=columns_to_drop)
 
     def get_test_run_result(self, version: Optional[str] = None) -> pd.DataFrame:
-        """Return test run metadata in a single-row Pandas dataframe."""
+        """Return test run metadata in a single-row Pandas dataframe.
+
+        Note: this will not work for test runs run on RIME versions <0.14.0
+        """
         if version and not VersionInfo.isvalid(version):
             raise ValueError(f"Invalid version string: {version}")
 
@@ -380,6 +400,138 @@ class RIMEProject(NamedTuple):
     description: str
 
 
+class RIMECTInstance:
+    """CTInstance object wrapper with helpful methods for working with RIME Continuous Testing."""
+
+    def __init__(self, backend: RIMEBackend, ct_instance_id: str,) -> None:
+        """Create a new CTInstance wrapper object.
+
+        Args:
+            backend: RIMEBackend
+                The RIME backend used to query about the status of the job.
+            ct_instance_id: str
+                The identifier for the RIME job that this object monitors.
+        """
+        self._backend = backend
+        self._ct_instance_id = ct_instance_id
+
+    def __eq__(self, obj: Any) -> bool:
+        """Check if this CTInstance is equivalent to 'obj'."""
+        return (
+            isinstance(obj, RIMECTInstance)
+            and self._ct_instance_id == obj._ct_instance_id
+        )
+
+    def _update_ct_instance(self, **update_params: Any) -> UpdateCTInstanceResponse:
+        req = UpdateCTInstanceRequest()
+        ct_instance_mask_params = {}
+        req.ct_instance.CopyFrom(CTInstance(id=self._ct_instance_id, **update_params))
+        for key in update_params:
+            ct_instance_mask_params[key] = True
+        req.mask.CopyFrom(CTInstanceWriteMask(**ct_instance_mask_params))
+        try:
+            with self._backend.get_continuous_testing_stub() as continuous_tester:
+                res = continuous_tester.UpdateCTInstance(req)
+                return res
+        except grpc.RpcError as e:
+            raise ValueError(e)
+
+    def activate_ct_instance(self) -> UpdateCTInstanceResponse:
+        """Set a CT instance as active.
+
+        Returns:
+            None
+
+        Raises:
+            ValueError
+                If the provided status_filters array has invalid values.
+                If the request to the ModelTest service failed.
+        """
+        return self._update_ct_instance(is_active=True)
+
+    def deactivate_ct_instance(self) -> UpdateCTInstanceResponse:
+        """Set a CT instance as inactive.
+
+        Returns:
+            None
+        """
+        return self._update_ct_instance(is_active=False)
+
+    def start_incremental_continuous_test(
+        self,
+        test_run_config: dict,
+        custom_image: Optional[CustomImage] = None,
+        rime_managed_image: Optional[str] = None,
+        ram_request_megabytes: Optional[int] = None,
+        cpu_request_millicores: Optional[int] = None,
+    ) -> RIMEStressTestJob:
+        """Start a RIME model continuous test on the backend's ModelTesting service.
+
+        Args:
+            test_run_config: dict
+                Configuration for the test to be run, which specifies paths to
+                the model and datasets to used for the test.
+            custom_image: Optional[CustomImage]
+                Specification of a customized container image to use running the model
+                test. The image must have all dependencies required by your model.
+                The image must specify a name for the image and optional a pull secret
+                (of type CustomImage.PullSecret) with the name of the kubernetes pull
+                secret used to access the given image.
+            rime_managed_image: Optional[str]
+                Name of a managed image to use when running the model test.
+                The image must have all dependencies required by your model. To create
+                new managed images with your desired dependencies, use the client's
+                `create_managed_image()` method.
+            ram_request_megabytes: int
+                Megabytes of RAM requested for the stress test job.
+                The limit is 2x the megabytes requested.
+            cpu_request_millicores: int
+                Millicores of CPU requested for the stress test job.
+                The limit is 2x the millicores requested.
+
+        Returns:
+            A RIMEStressTestJob providing information about the model stress test job.
+
+        Raises:
+            ValueError
+                If the request to the ModelTest service failed.
+
+        TODO(blaine): Add config validation service.
+        """
+        if not isinstance(test_run_config, dict):
+            raise ValueError("The configuration must be a dictionary")
+
+        if custom_image and rime_managed_image:
+            raise ValueError(
+                "Cannot specify both 'custom_image' and 'rime_managed_image'"
+            )
+
+        req = StartIncrementalContinuousTestRequest(
+            ct_instance_id=self._ct_instance_id,
+            test_run_config=simplejson.dumps(test_run_config).encode(),
+        )
+        if custom_image:
+            req.custom_image_type.testing_image.CopyFrom(custom_image)
+        if rime_managed_image:
+            req.custom_image_type.managed_image.name = rime_managed_image
+        if ram_request_megabytes:
+            req.ram_request_megabytes = ram_request_megabytes
+        if cpu_request_millicores:
+            req.cpu_request_millicores = cpu_request_millicores
+        try:
+            RIMEClient._throttler.throttle(  # pylint: disable=W0212
+                throttling_msg="Your request is throttled to limit # of model tests."
+            )
+            with self._backend.get_model_testing_stub() as model_tester:
+                job: JobMetadata = model_tester.StartIncrementalContinuousTest(
+                    request=req
+                ).job
+                return RIMEStressTestJob(self._backend, job.id)
+        except grpc.RpcError as e:
+            # TODO(blaine): differentiate on different error types.
+            raise ValueError(e)
+
+
 class RIMEClient:
     """RIMEClient provides an interface to RIME backend services."""
 
@@ -583,6 +735,8 @@ class RIMEClient:
         project_id: Optional[str] = None,
         custom_image: Optional[CustomImage] = None,
         rime_managed_image: Optional[str] = None,
+        ram_request_megabytes: Optional[int] = None,
+        cpu_request_millicores: Optional[int] = None,
     ) -> RIMEStressTestJob:
         """Start a RIME model stress test on the backend's ModelTesting service.
 
@@ -604,6 +758,12 @@ class RIMEClient:
                 The image must have all dependencies required by your model. To create
                 new managed images with your desired dependencies, use the client's
                 `create_managed_image()` method.
+            ram_request_megabytes: int
+                Megabytes of RAM requested for the stress test job.
+                The limit is 2x the megabytes requested.
+            cpu_request_millicores: int
+                Millicores of CPU requested for the stress test job.
+                The limit is 2x the millicores requested.
 
         Returns:
             A RIMEStressTestJob providing information about the model stress test job.
@@ -634,6 +794,10 @@ class RIMEClient:
             req.testing_image.CopyFrom(custom_image)
         if rime_managed_image:
             req.managed_image.name = rime_managed_image
+        if ram_request_megabytes:
+            req.ram_request_megabytes = ram_request_megabytes
+        if cpu_request_millicores:
+            req.cpu_request_millicores = cpu_request_millicores
         try:
             RIMEClient._throttler.throttle(
                 throttling_msg="Your request is throttled to limit # of model tests."
@@ -650,7 +814,7 @@ class RIMEClient:
         status_filters: Optional[List[str]] = None,
         project_id: Optional[str] = None,
     ) -> List[RIMEStressTestJob]:
-        """Query the ModelTest service for a list of jobs.
+        """Query the ModelTest service for a list of stress test jobs.
 
         Args:
             status_filters: Optional[List[str]]
@@ -684,9 +848,177 @@ class RIMEClient:
             raise ValueError("Project id {} does not exist".format(project_id))
         if project_id:
             req.project_id = project_id
+        # Filter only for stress testing jobs.
+        req.selected_types.extend([JobType.MODEL_STRESS_TEST])
         try:
             with self._backend.get_model_testing_stub() as model_tester:
                 res = model_tester.ListTestJobs(req)
                 return [RIMEStressTestJob(self._backend, job.id) for job in res.jobs]
         except grpc.RpcError as e:
             raise ValueError(e)
+
+    def create_ct_instance(
+        self, name: str, bin_size_seconds: int, ot_test_run_id: str, project_id: str
+    ) -> RIMECTInstance:
+        """Create a CT Instance using the CT service.
+
+        Args:
+            name: str
+                CT instance name.
+            bin_size_seconds: int
+                Bin size in seconds. Only supports daily or hourly.
+            ot_test_run_id: str
+                ID of the offline test run that continuous tests will be based on.
+            project_id: str
+                ID of the project this CT instance belongs to.
+
+        Returns:
+            ID of the CT Instance created
+
+        Raises:
+            ValueError
+                If the provided status_filters array has invalid values.
+                If the request to the ModelTest service failed.
+        """
+        req = CreateCTInstanceRequest()
+        req.ct_instance.CopyFrom(
+            CTInstance(
+                name=name,
+                bin_size_seconds=bin_size_seconds,
+                ot_test_run_id=ot_test_run_id,
+                project_id=project_id,
+            )
+        )
+        try:
+            with self._backend.get_continuous_testing_stub() as continuous_tester:
+                res: CreateCTInstanceResponse = continuous_tester.CreateCTInstance(req)
+                return RIMECTInstance(self._backend, res.ct_instance_id)
+        except grpc.RpcError as e:
+            raise ValueError(e)
+
+    def get_ct_instance(self, ct_instance_id: str) -> RIMECTInstance:
+        """Get a ct instance if it exists.
+
+        Args:
+            ct_instance_id: ID of the CT instance to fetch.
+
+        Returns:
+            CTInstance Object
+
+        Raises:
+            ValueError
+                If the CT Instance does not exist.
+        """
+        req = GetCTInstanceRequest()
+        req.ct_instance_id = ct_instance_id
+        try:
+            with self._backend.get_continuous_testing_stub() as continuous_tester:
+                res: GetCTInstanceResponse = continuous_tester.GetCTInstance(req)
+                return RIMECTInstance(self._backend, res.ct_instance.id)
+        except grpc.RpcError as e:
+            raise ValueError(e)
+
+    def get_active_ct_instance_for_project(self, project_id: str) -> RIMECTInstance:
+        """Get the active ct instance for a project if it exists.
+
+        Args:
+            project_id: Project containing an active ct instance id.
+
+        Returns:
+            CTInstance Object
+
+        Raises:
+            ValueError
+                If the CT Instance does not exist.
+        """
+        req = GetActiveCTInstanceIDRequest(project_id=project_id)
+        try:
+            with self._backend.get_continuous_testing_stub() as continuous_tester:
+                res: GetActiveCTInstanceIDResponse = continuous_tester.GetActiveCTInstanceID(  # pylint: disable=C0301
+                    req
+                )
+                return RIMECTInstance(self._backend, res.ct_instance_id)
+        except grpc.RpcError as e:
+            raise ValueError(e)
+
+    def start_continuous_test_from_reference(
+        self,
+        test_run_config: dict,
+        project_id: Optional[str] = None,
+        custom_image: Optional[CustomImage] = None,
+        rime_managed_image: Optional[str] = None,
+        ram_request_megabytes: Optional[int] = None,
+        cpu_request_millicores: Optional[int] = None,
+    ) -> RIMEStressTestJob:
+        """Start a RIME continuous test from reference on the backend's ModelTesting service.
+
+        Args:
+            test_run_config: dict
+                Configuration for the test to be run, which specifies paths to
+                the model and datasets to used for the test.
+            project_id: Optional[str]
+                Identifier for the project where the resulting test run will be stored.
+                If not specified, the results will be stored in the default project.
+            custom_image: Optional[CustomImage]
+                Specification of a customized container image to use running the model
+                test. The image must have all dependencies required by your model.
+                The image must specify a name for the image and optional a pull secret
+                (of type CustomImage.PullSecret) with the name of the kubernetes pull
+                secret used to access the given image.
+            rime_managed_image: Optional[str]
+                Name of a managed image to use when running the model test.
+                The image must have all dependencies required by your model. To create
+                new managed images with your desired dependencies, use the client's
+                `create_managed_image()` method.
+            ram_request_megabytes: int
+                Megabytes of RAM requested for the stress test job.
+                The limit is 2x the megabytes requested.
+            cpu_request_millicores: int
+                Millicores of CPU requested for the stress test job.
+                The limit is 2x the millicores requested.
+
+        Returns:
+            A RIMEStressTestJob providing information about the model stress test job.
+
+        Raises:
+            ValueError
+                If the request to the ModelTest service failed.
+
+        TODO(blaine): Add config validation service.
+        """
+        if not isinstance(test_run_config, dict):
+            raise ValueError("The configuration must be a dictionary")
+
+        if custom_image and rime_managed_image:
+            raise ValueError(
+                "Cannot specify both 'custom_image' and 'rime_managed_image'"
+            )
+
+        if project_id and not self._project_exists(project_id):
+            raise ValueError("Project id {} does not exist".format(project_id))
+
+        req = StartContinuousTestFromReferenceRequest(
+            test_run_config=simplejson.dumps(test_run_config).encode(),
+        )
+        if project_id:
+            req.project_id = project_id
+        if custom_image:
+            req.custom_image_type.testing_image.CopyFrom(custom_image)
+        if rime_managed_image:
+            req.custom_image_type.managed_image.name = rime_managed_image
+        if ram_request_megabytes:
+            req.ram_request_megabytes = ram_request_megabytes
+        if cpu_request_millicores:
+            req.cpu_request_millicores = cpu_request_millicores
+        try:
+            RIMEClient._throttler.throttle(
+                throttling_msg="Your request is throttled to limit # of model tests."
+            )
+            with self._backend.get_model_testing_stub() as model_tester:
+                job: JobMetadata = model_tester.StartContinuousTestFromReference(
+                    request=req
+                ).job
+                return RIMEStressTestJob(self._backend, job.id)
+        except grpc.RpcError as e:
+            # TODO(blaine): differentiate on different error types.
+            raise ValueError(e)
