@@ -4,6 +4,7 @@ from typing import Any, Callable, Generic, Optional, TypeVar
 
 import grpc
 
+from rime_sdk.protos.continuous.continuous_pb2_grpc import CTServiceStub
 from rime_sdk.protos.image_registry.image_registry_pb2_grpc import ImageRegistryStub
 from rime_sdk.protos.model_testing.model_testing_pb2_grpc import ModelTestingStub
 from rime_sdk.protos.results_upload.results_upload_pb2_grpc import ResultsStoreStub
@@ -139,6 +140,7 @@ class RIMEBackend:
         self._results_store_addr = self._get_results_store_addr(base_domain)
         self._test_run_tracker_addr = self._get_test_run_tracker_addr(base_domain)
         self._test_run_results_addr = self._get_test_run_results_addr(base_domain)
+        self._grpc_web_addr = self._get_continuous_addr(base_domain)
 
     def _get_image_registry_addr(self, domain: str) -> str:
         """Construct an address to the image-registry service from `domain`.
@@ -185,6 +187,15 @@ class RIMEBackend:
         """
         return f"rime-test-run-results.{domain}:443"
 
+    def _get_continuous_addr(self, domain: str) -> str:
+        """Construct an address to the continuous service from `domain`.
+
+        Args:
+            domain: str
+                The base domain/address of the RIME service.
+        """
+        return f"rime-continuous.{domain}:443"
+
     def get_image_registry_stub(self) -> RIMEConnection[ImageRegistryStub]:
         """Return an image registry client."""
         return RIMEConnection[ImageRegistryStub](
@@ -229,3 +240,12 @@ class RIMEBackend:
             self._api_key,
             channel_timeout=self._channel_timeout,
         )
+
+    def get_continuous_testing_stub(self) -> RIMEConnection[CTServiceStub]:
+        """Return a continuous testing client."""
+        return RIMEConnection[CTServiceStub](
+            CTServiceStub,
+            self._grpc_web_addr,
+            self._api_key,
+            channel_timeout=self._channel_timeout,
+        )
